@@ -7,12 +7,16 @@ Test cases for L{twisted.internet.defer}.
 
 from __future__ import division, absolute_import
 
+import warnings
 import gc, traceback
+import re
 
-from twisted.python.compat import _PY3
-from twisted.trial import unittest
-from twisted.internet import defer
 from twisted.python import failure, log
+from twisted.python.compat import _PY3
+from twisted.internet import defer, reactor
+from twisted.internet.task import Clock
+from twisted.trial import unittest
+
 
 
 class GenericError(Exception):
@@ -35,6 +39,17 @@ def getDivisionFailure(*args, **kwargs):
 
 
 
+def fakeCallbackCanceller(deferred):
+    """
+    A fake L{defer.Deferred} canceller which callbacks the L{defer.Deferred}
+    with C{str} "Callback Result" when cancelling it.
+
+    @param deferred: The cancelled L{defer.Deferred}.
+    """
+    deferred.callback("Callback Result")
+
+
+
 class ImmediateFailureMixin(object):
     """
     Add additional assertion methods.
@@ -55,7 +70,53 @@ class ImmediateFailureMixin(object):
 
 
 
-class DeferredTestCase(unittest.SynchronousTestCase, ImmediateFailureMixin):
+class UtilTests(unittest.TestCase):
+    """
+    Tests for utility functions.
+    """
+
+    def test_logErrorReturnsError(self):
+        """
+        L{defer.logError} returns the given error.
+        """
+        error = failure.Failure(RuntimeError())
+        result = defer.logError(error)
+        self.flushLoggedErrors(RuntimeError)
+
+        self.assertIs(error, result)
+
+
+    def test_logErrorLogsError(self):
+        """
+        L{defer.logError} logs the given error.
+        """
+        error = failure.Failure(RuntimeError())
+        defer.logError(error)
+        errors = self.flushLoggedErrors(RuntimeError)
+
+        self.assertEquals(errors, [error])
+
+
+    def test_logErrorLogsErrorNoRepr(self):
+        """
+        The text logged by L{defer.logError} has no repr of the failure.
+        """
+        output = []
+
+        def emit(eventDict):
+            output.append(log.textFromEventDict(eventDict))
+
+        log.addObserver(emit)
+
+        error = failure.Failure(RuntimeError())
+        defer.logError(error)
+        self.flushLoggedErrors(RuntimeError)
+
+        self.assertTrue(output[0].startswith("Unhandled Error\nTraceback "))
+
+
+
+class DeferredTests(unittest.SynchronousTestCase, ImmediateFailureMixin):
 
     def setUp(self):
         self.callbackResults = None
@@ -253,6 +314,177 @@ class DeferredTestCase(unittest.SynchronousTestCase, ImmediateFailureMixin):
         d1.addErrback(lambda e: None)  # Swallow error
 
 
+    def test_cancelDeferredList(self):
+        """
+        When cancelling an unfired L{defer.DeferredList}, cancel every
+        L{defer.Deferred} in the list.
+        """
+        deferredOne = defer.Deferred()
+        deferredTwo = defer.Deferred()
+        deferredList = defer.DeferredList([deferredOne, deferredTwo])
+        deferredList.cancel()
+        self.failureResultOf(deferredOne, defer.CancelledError)
+        self.failureResultOf(deferredTwo, defer.CancelledError)
+
+
+    def test_cancelDeferredListCallback(self):
+        """
+        When cancelling an unfired L{defer.DeferredList} without the
+        C{fireOnOneCallback} and C{fireOnOneErrback} flags set, the
+        L{defer.DeferredList} will be callback with a C{list} of
+        (success, result) C{tuple}s.
+        """
+        deferredOne = defer.Deferred(fakeCallbackCanceller)
+        deferredTwo = defer.Deferred()
+        deferredList = defer.DeferredList([deferredOne, deferredTwo])
+        deferredList.cancel()
+        self.failureResultOf(deferredTwo, defer.CancelledError)
+        result = self.successResultOf(deferredList)
+        self.assertTrue(result[0][0])
+        self.assertEqual(result[0][1], "Callback Result")
+        self.assertFalse(result[1][0])
+        self.assertTrue(result[1][1].check(defer.CancelledError))
+
+
+    def test_cancelDeferredListWithFireOnOneCallback(self):
+        """
+        When cancelling an unfired L{defer.DeferredList} with the flag
+        C{fireOnOneCallback} set, cancel every L{defer.Deferred} in the list.
+        """
+        deferredOne = defer.Deferred()
+        deferredTwo = defer.Deferred()
+        deferredList = defer.DeferredList([deferredOne, deferredTwo],
+                                          fireOnOneCallback=True)
+        deferredList.cancel()
+        self.failureResultOf(deferredOne, defer.CancelledError)
+        self.failureResultOf(deferredTwo, defer.CancelledError)
+
+
+    def test_cancelDeferredListWithFireOnOneCallbackAndDeferredCallback(self):
+        """
+        When cancelling an unfired L{defer.DeferredList} with the flag
+        C{fireOnOneCallback} set, if one of the L{defer.Deferred} callbacks
+        in its canceller, the L{defer.DeferredList} will callback with the
+        result and the index of the L{defer.Deferred} in a C{tuple}.
+        """
+        deferredOne = defer.Deferred(fakeCallbackCanceller)
+        deferredTwo = defer.Deferred()
+        deferredList = defer.DeferredList([deferredOne, deferredTwo],
+                                          fireOnOneCallback=True)
+        deferredList.cancel()
+        self.failureResultOf(deferredTwo, defer.CancelledError)
+        result = self.successResultOf(deferredList)
+        self.assertEqual(result, ("Callback Result", 0))
+
+
+    def test_cancelDeferredListWithFireOnOneErrback(self):
+        """
+        When cancelling an unfired L{defer.DeferredList} with the flag
+        C{fireOnOneErrback} set, cancel every L{defer.Deferred} in the list.
+        """
+        deferredOne = defer.Deferred()
+        deferredTwo = defer.Deferred()
+        deferredList = defer.DeferredList([deferredOne, deferredTwo],
+                                          fireOnOneErrback=True)
+        deferredList.cancel()
+        self.failureResultOf(deferredOne, defer.CancelledError)
+        self.failureResultOf(deferredTwo, defer.CancelledError)
+        deferredListFailure = self.failureResultOf(deferredList,
+                                                   defer.FirstError)
+        firstError = deferredListFailure.value
+        self.assertTrue(firstError.subFailure.check(defer.CancelledError))
+
+
+    def test_cancelDeferredListWithFireOnOneErrbackAllDeferredsCallback(self):
+        """
+        When cancelling an unfired L{defer.DeferredList} with the flag
+        C{fireOnOneErrback} set, if all the L{defer.Deferred} callbacks
+        in its canceller, the L{defer.DeferredList} will callback with a
+        C{list} of (success, result) C{tuple}s.
+        """
+        deferredOne = defer.Deferred(fakeCallbackCanceller)
+        deferredTwo = defer.Deferred(fakeCallbackCanceller)
+        deferredList = defer.DeferredList([deferredOne, deferredTwo],
+                                          fireOnOneErrback=True)
+        deferredList.cancel()
+        result = self.successResultOf(deferredList)
+        self.assertTrue(result[0][0])
+        self.assertEqual(result[0][1], "Callback Result")
+        self.assertTrue(result[1][0])
+        self.assertEqual(result[1][1], "Callback Result")
+
+
+    def test_cancelDeferredListWithOriginalDeferreds(self):
+        """
+        Cancelling a L{defer.DeferredList} will cancel the original
+        L{defer.Deferred}s passed in.
+        """
+        deferredOne = defer.Deferred()
+        deferredTwo = defer.Deferred()
+        argumentList = [deferredOne, deferredTwo]
+        deferredList = defer.DeferredList(argumentList)
+        deferredThree = defer.Deferred()
+        argumentList.append(deferredThree)
+        deferredList.cancel()
+        self.failureResultOf(deferredOne, defer.CancelledError)
+        self.failureResultOf(deferredTwo, defer.CancelledError)
+        self.assertNoResult(deferredThree)
+
+
+    def test_cancelDeferredListWithException(self):
+        """
+        Cancelling a L{defer.DeferredList} will cancel every L{defer.Deferred}
+        in the list even exceptions raised from the C{cancel} method of the
+        L{defer.Deferred}s.
+        """
+        def cancellerRaisesException(deferred):
+            """
+            A L{defer.Deferred} canceller that raises an exception.
+
+            @param deferred: The cancelled L{defer.Deferred}.
+            """
+            raise RuntimeError("test")
+        deferredOne = defer.Deferred(cancellerRaisesException)
+        deferredTwo = defer.Deferred()
+        deferredList = defer.DeferredList([deferredOne, deferredTwo])
+        deferredList.cancel()
+        self.failureResultOf(deferredTwo, defer.CancelledError)
+        errors = self.flushLoggedErrors(RuntimeError)
+        self.assertEqual(len(errors), 1)
+
+
+    def test_cancelFiredOnOneCallbackDeferredList(self):
+        """
+        When a L{defer.DeferredList} has fired because one L{defer.Deferred} in
+        the list fired with a non-failure result, the cancellation will do
+        nothing instead of cancelling the rest of the L{defer.Deferred}s.
+        """
+        deferredOne = defer.Deferred()
+        deferredTwo = defer.Deferred()
+        deferredList = defer.DeferredList([deferredOne, deferredTwo],
+                                          fireOnOneCallback=True)
+        deferredOne.callback(None)
+        deferredList.cancel()
+        self.assertNoResult(deferredTwo)
+
+
+    def test_cancelFiredOnOneErrbackDeferredList(self):
+        """
+        When a L{defer.DeferredList} has fired because one L{defer.Deferred} in
+        the list fired with a failure result, the cancellation will do
+        nothing instead of cancelling the rest of the L{defer.Deferred}s.
+        """
+        deferredOne = defer.Deferred()
+        deferredTwo = defer.Deferred()
+        deferredList = defer.DeferredList([deferredOne, deferredTwo],
+                                          fireOnOneErrback=True)
+        deferredOne.errback(GenericError("test"))
+        deferredList.cancel()
+        self.assertNoResult(deferredTwo)
+        self.failureResultOf(deferredOne, GenericError)
+        self.failureResultOf(deferredList, defer.FirstError)
+
+
     def testImmediateSuccess(self):
         l = []
         d = defer.succeed("success")
@@ -381,6 +613,38 @@ class DeferredTestCase(unittest.SynchronousTestCase, ImmediateFailureMixin):
         self.assertIsInstance(firstError.value, RuntimeError)
 
 
+    def test_cancelGatherResults(self):
+        """
+        When cancelling the L{defer.gatherResults} call, all the
+        L{defer.Deferred}s in the list will be cancelled.
+        """
+        deferredOne = defer.Deferred()
+        deferredTwo = defer.Deferred()
+        result = defer.gatherResults([deferredOne, deferredTwo])
+        result.cancel()
+        self.failureResultOf(deferredOne, defer.CancelledError)
+        self.failureResultOf(deferredTwo, defer.CancelledError)
+        gatherResultsFailure = self.failureResultOf(result, defer.FirstError)
+        firstError = gatherResultsFailure.value
+        self.assertTrue(firstError.subFailure.check(defer.CancelledError))
+
+
+    def test_cancelGatherResultsWithAllDeferredsCallback(self):
+        """
+        When cancelling the L{defer.gatherResults} call, if all the
+        L{defer.Deferred}s callback in their canceller, the L{defer.Deferred}
+        returned by L{defer.gatherResults} will be callbacked with the C{list}
+        of the results.
+        """
+        deferredOne = defer.Deferred(fakeCallbackCanceller)
+        deferredTwo = defer.Deferred(fakeCallbackCanceller)
+        result = defer.gatherResults([deferredOne, deferredTwo])
+        result.cancel()
+        callbackResult = self.successResultOf(result)
+        self.assertEqual(callbackResult[0], "Callback Result")
+        self.assertEqual(callbackResult[1], "Callback Result")
+
+
     def test_maybeDeferredSync(self):
         """
         L{defer.maybeDeferred} should retrieve the result of a synchronous
@@ -900,6 +1164,44 @@ class DeferredTestCase(unittest.SynchronousTestCase, ImmediateFailureMixin):
         self.assertIdentical(a._chainedTo, b)
 
 
+    def test_circularChainWarning(self):
+        """
+        When a Deferred is returned from a callback directly attached to that
+        same Deferred, a warning is emitted.
+        """
+        d = defer.Deferred()
+        def circularCallback(result):
+            return d
+        d.addCallback(circularCallback)
+        d.callback("foo")
+
+        circular_warnings = self.flushWarnings([circularCallback])
+        self.assertEqual(len(circular_warnings), 1)
+        warning = circular_warnings[0]
+        self.assertEqual(warning['category'], DeprecationWarning)
+        pattern = "Callback returned the Deferred it was attached to"
+        self.assertTrue(
+            re.search(pattern, warning['message']),
+            "\nExpected match: %r\nGot: %r" % (pattern, warning['message']))
+
+
+    def test_circularChainException(self):
+        """
+        If the deprecation warning for circular deferred callbacks is
+        configured to be an error, the exception will become the failure
+        result of the Deferred.
+        """
+        self.addCleanup(setattr, warnings, "filters", warnings.filters)
+        warnings.filterwarnings("error", category=DeprecationWarning)
+        d = defer.Deferred()
+        def circularCallback(result):
+            return d
+        d.addCallback(circularCallback)
+        d.callback("foo")
+        failure = self.failureResultOf(d)
+        failure.trap(DeprecationWarning)
+
+
     def test_repr(self):
         """
         The C{repr()} of a L{Deferred} contains the class name and a
@@ -1114,10 +1416,9 @@ class DeferredTestCase(unittest.SynchronousTestCase, ImmediateFailureMixin):
 
 
     if _PY3:
-        test_inlineCallbacksTracebacks.todo = (
+        # FIXME: https://twistedmatrix.com/trac/ticket/5949
+        test_inlineCallbacksTracebacks.skip = (
             "Python 3 support to be fixed in #5949")
-        # Remove this line in #6008 (unittest todo support):
-        del test_inlineCallbacksTracebacks
 
 
 
@@ -1189,7 +1490,7 @@ class FirstErrorTests(unittest.SynchronousTestCase):
 
 
 
-class AlreadyCalledTestCase(unittest.SynchronousTestCase):
+class AlreadyCalledTests(unittest.SynchronousTestCase):
     def setUp(self):
         self._deferredWasDebugging = defer.getDebugging()
         defer.setDebugging(True)
@@ -1332,7 +1633,7 @@ class AlreadyCalledTestCase(unittest.SynchronousTestCase):
 
 
 
-class DeferredCancellerTest(unittest.SynchronousTestCase):
+class DeferredCancellerTests(unittest.SynchronousTestCase):
     def setUp(self):
         self.callbackResults = None
         self.errbackResults = None
@@ -1595,7 +1896,7 @@ class DeferredCancellerTest(unittest.SynchronousTestCase):
 
 
 
-class LogTestCase(unittest.SynchronousTestCase):
+class LogTests(unittest.SynchronousTestCase):
     """
     Test logging of unhandled errors.
     """
@@ -1665,6 +1966,48 @@ class LogTestCase(unittest.SynchronousTestCase):
         self._check()
 
 
+    def test_errorLogNoRepr(self):
+        """
+        Verify that when a L{Deferred} with no references to it is fired,
+        the logged message does not contain a repr of the failure object.
+        """
+        defer.Deferred().addCallback(lambda x: 1 // 0).callback(1)
+
+        gc.collect()
+        self._check()
+
+        self.assertEqual(2, len(self.c))
+        msg = log.textFromEventDict(self.c[-1])
+        expected = "Unhandled Error\nTraceback "
+        self.assertTrue(msg.startswith(expected),
+                        "Expected message starting with: {0!r}".
+                            format(expected))
+
+
+    def test_errorLogDebugInfo(self):
+        """
+        Verify that when a L{Deferred} with no references to it is fired,
+        the logged message includes debug info if debugging on the deferred
+        is enabled.
+        """
+        def doit():
+            d = defer.Deferred()
+            d.debug = True
+            d.addCallback(lambda x: 1 // 0)
+            d.callback(1)
+
+        doit()
+        gc.collect()
+        self._check()
+
+        self.assertEqual(2, len(self.c))
+        msg = log.textFromEventDict(self.c[-1])
+        expected = "(debug:  I"
+        self.assertTrue(msg.startswith(expected),
+                        "Expected message starting with: {0!r}".
+                            format(expected))
+
+
     def test_chainedErrorCleanup(self):
         """
         If one Deferred with an error result is returned from a callback on
@@ -1713,7 +2056,7 @@ class LogTestCase(unittest.SynchronousTestCase):
 
 
 
-class DeferredTestCaseII(unittest.SynchronousTestCase):
+class DeferredListEmptyTests(unittest.SynchronousTestCase):
     def setUp(self):
         self.callbackRan = 0
 
@@ -1731,7 +2074,7 @@ class DeferredTestCaseII(unittest.SynchronousTestCase):
 
 
 
-class OtherPrimitives(unittest.SynchronousTestCase, ImmediateFailureMixin):
+class OtherPrimitivesTests(unittest.SynchronousTestCase, ImmediateFailureMixin):
     def _incr(self, result):
         self.counter += 1
 
@@ -1972,108 +2315,134 @@ class OtherPrimitives(unittest.SynchronousTestCase, ImmediateFailureMixin):
         self.assertEqual(len(done), 1)
 
 
-# Enable on Python 3 as part of #5960:
-if not _PY3:
-    from twisted.internet import reactor
-    from twisted.internet.task import Clock
 
-    class DeferredFilesystemLockTestCase(unittest.TestCase):
+class DeferredFilesystemLockTests(unittest.TestCase):
+    """
+    Test the behavior of L{DeferredFilesystemLock}
+    """
+
+    def setUp(self):
+        self.clock = Clock()
+        self.lock = defer.DeferredFilesystemLock(self.mktemp(),
+                                                 scheduler=self.clock)
+
+
+    def test_waitUntilLockedWithNoLock(self):
         """
-        Test the behavior of L{DeferredFilesystemLock}
+        Test that the lock can be acquired when no lock is held
         """
+        d = self.lock.deferUntilLocked(timeout=1)
 
-        def setUp(self):
-            self.clock = Clock()
-            self.lock = defer.DeferredFilesystemLock(self.mktemp(),
-                                                     scheduler=self.clock)
+        return d
 
 
-        def test_waitUntilLockedWithNoLock(self):
-            """
-            Test that the lock can be acquired when no lock is held
-            """
-            d = self.lock.deferUntilLocked(timeout=1)
+    def test_waitUntilLockedWithTimeoutLocked(self):
+        """
+        Test that the lock can not be acquired when the lock is held
+        for longer than the timeout.
+        """
+        self.failUnless(self.lock.lock())
 
-            return d
+        d = self.lock.deferUntilLocked(timeout=5.5)
+        self.assertFailure(d, defer.TimeoutError)
+
+        self.clock.pump([1] * 10)
+
+        return d
 
 
-        def test_waitUntilLockedWithTimeoutLocked(self):
-            """
-            Test that the lock can not be acquired when the lock is held
-            for longer than the timeout.
-            """
-            self.failUnless(self.lock.lock())
+    def test_waitUntilLockedWithTimeoutUnlocked(self):
+        """
+        Test that a lock can be acquired while a lock is held
+        but the lock is unlocked before our timeout.
+        """
+        def onTimeout(f):
+            f.trap(defer.TimeoutError)
+            self.fail("Should not have timed out")
 
-            d = self.lock.deferUntilLocked(timeout=5.5)
-            self.assertFailure(d, defer.TimeoutError)
+        self.failUnless(self.lock.lock())
 
-            self.clock.pump([1] * 10)
+        self.clock.callLater(1, self.lock.unlock)
+        d = self.lock.deferUntilLocked(timeout=10)
+        d.addErrback(onTimeout)
 
-            return d
+        self.clock.pump([1] * 10)
+
+        return d
 
 
-        def test_waitUntilLockedWithTimeoutUnlocked(self):
-            """
-            Test that a lock can be acquired while a lock is held
-            but the lock is unlocked before our timeout.
-            """
-            def onTimeout(f):
-                f.trap(defer.TimeoutError)
-                self.fail("Should not have timed out")
+    def test_defaultScheduler(self):
+        """
+        Test that the default scheduler is set up properly.
+        """
+        lock = defer.DeferredFilesystemLock(self.mktemp())
 
-            self.failUnless(self.lock.lock())
-
-            self.clock.callLater(1, self.lock.unlock)
-            d = self.lock.deferUntilLocked(timeout=10)
-            d.addErrback(onTimeout)
-
-            self.clock.pump([1] * 10)
-
-            return d
+        self.assertEqual(lock._scheduler, reactor)
 
 
-        def test_defaultScheduler(self):
-            """
-            Test that the default scheduler is set up properly.
-            """
-            lock = defer.DeferredFilesystemLock(self.mktemp())
+    def test_concurrentUsage(self):
+        """
+        Test that an appropriate exception is raised when attempting
+        to use deferUntilLocked concurrently.
+        """
+        self.lock.lock()
+        self.clock.callLater(1, self.lock.unlock)
 
-            self.assertEqual(lock._scheduler, reactor)
+        d = self.lock.deferUntilLocked()
+        d2 = self.lock.deferUntilLocked()
+
+        self.assertFailure(d2, defer.AlreadyTryingToLockError)
+
+        self.clock.advance(1)
+
+        return d
 
 
-        def test_concurrentUsage(self):
-            """
-            Test that an appropriate exception is raised when attempting
-            to use deferUntilLocked concurrently.
-            """
-            self.lock.lock()
-            self.clock.callLater(1, self.lock.unlock)
-
+    def test_multipleUsages(self):
+        """
+        Test that a DeferredFilesystemLock can be used multiple times
+        """
+        def lockAquired(ign):
+            self.lock.unlock()
             d = self.lock.deferUntilLocked()
-            d2 = self.lock.deferUntilLocked()
-
-            self.assertFailure(d2, defer.AlreadyTryingToLockError)
-
-            self.clock.advance(1)
-
             return d
 
+        self.lock.lock()
+        self.clock.callLater(1, self.lock.unlock)
 
-        def test_multipleUsages(self):
-            """
-            Test that a DeferredFilesystemLock can be used multiple times
-            """
-            def lockAquired(ign):
-                self.lock.unlock()
-                d = self.lock.deferUntilLocked()
-                return d
+        d = self.lock.deferUntilLocked()
+        d.addCallback(lockAquired)
 
-            self.lock.lock()
-            self.clock.callLater(1, self.lock.unlock)
+        self.clock.advance(1)
 
-            d = self.lock.deferUntilLocked()
-            d.addCallback(lockAquired)
+        return d
 
-            self.clock.advance(1)
 
-            return d
+    def test_cancelDeferUntilLocked(self):
+        """
+        When cancelling a L{defer.Deferred} returned by
+        L{defer.DeferredFilesystemLock.deferUntilLocked}, the
+        L{defer.DeferredFilesystemLock._tryLockCall} is cancelled.
+        """
+        self.lock.lock()
+        deferred = self.lock.deferUntilLocked()
+        tryLockCall = self.lock._tryLockCall
+        deferred.cancel()
+        self.assertFalse(tryLockCall.active())
+        self.assertEqual(self.lock._tryLockCall, None)
+        self.failureResultOf(deferred, defer.CancelledError)
+
+
+    def test_cancelDeferUntilLockedWithTimeout(self):
+        """
+        When cancel a L{defer.Deferred} returned by
+        L{defer.DeferredFilesystemLock.deferUntilLocked}, if the timeout is
+        set, the timeout call will be cancelled.
+        """
+        self.lock.lock()
+        deferred = self.lock.deferUntilLocked(timeout=1)
+        timeoutCall = self.lock._timeoutCall
+        deferred.cancel()
+        self.assertFalse(timeoutCall.active())
+        self.assertEqual(self.lock._timeoutCall, None)
+        self.failureResultOf(deferred, defer.CancelledError)
